import os
from typing import Any, Callable, Union
from unittest.mock import AsyncMock, MagicMock

from aiohttp import ClientResponse, ClientSession
import pytest

import pykoplenti

only_smoketest: pytest.MarkDecorator = pytest.mark.skipif(
    os.getenv("SMOKETEST_HOST") is None, reason="Smoketest must be explicitly executed"
)


@pytest.fixture
def websession_responses() -> list[MagicMock]:
    """Provides a mutable list for responses of a ClientSession."""
    return []


@pytest.fixture
def websession(websession_responses) -> MagicMock:
    """Creates a mocked ClientSession.

    The client_response_factoryfixture can be used to add responses.
    """
    websession = MagicMock(spec_set=ClientSession, name="websession Mock")
    websession.request.return_value.__aenter__.side_effect = websession_responses
    return websession


@pytest.fixture
def client_response_factory(
    websession_responses,
) -> Callable[[int, Any], MagicMock]:
    """Provides a factory to add responses to a ClientSession."""

    def factory(status: int = 200, json: Union[list[Any], dict[Any, Any], None] = None):
        response = MagicMock(spec_set=ClientResponse, name="request Mock")
        response.status = status
        if json is not None:
            response.json.return_value = json

        websession_responses.append(response)
        return response

    return factory


@pytest.fixture
def pykoplenti_client(websession) -> pykoplenti.ApiClient:
    """Returns a pykoplenti API-Client.

    The _login method is replaced with an AsyncMock.
    """
    client = pykoplenti.ApiClient(websession, "localhost")
    login_mock = AsyncMock()
    client._login = login_mock  # type: ignore

    return client


@pytest.fixture
def pykoplenti_extended_client(websession) -> pykoplenti.ExtendedApiClient:
    """Returns a pykoplenti Extended API-Client.

    The _login method is replaced with an AsyncMock.
    """
    client = pykoplenti.ExtendedApiClient(websession, "localhost")
    login_mock = AsyncMock()
    client._login = login_mock  # type: ignore

    return client


@pytest.fixture
def smoketest_config() -> tuple[str, int, str]:
    """Return the configuration for smoke tests."""
    return (
        os.getenv("SMOKETEST_HOST", "localhost"),
        int(os.getenv("SMOKETEST_PORT", 80)),
        os.getenv("SMOKETEST_PASS", ""),
    )



from pathlib import Path
from click.testing import CliRunner
import pytest
from pykoplenti.cli import cli, SessionCache
import os
from conftest import only_smoketest


@pytest.fixture
def credentials(tmp_path: Path, smoketest_config: tuple[str, int, str]):
    _, _, password = smoketest_config
    credentials_path = tmp_path / "credentials"
    credentials_path.write_text(f"password={password}")
    return credentials_path


@pytest.fixture
def dummy_credentials(tmp_path: Path):
    credentials_path = tmp_path / "credentials"
    credentials_path.write_text("password=dummy")
    return credentials_path


@pytest.fixture
def session_cache(smoketest_config: tuple[str, int, str]):
    host, _, _ = smoketest_config
    session_cache = SessionCache(host, "user")
    session_cache.remove()
    yield session_cache
    session_cache.remove()


class TestInvalidGlobalOptions:
    """Test invalid global options."""

    def test_crendentials_and_password(self, dummy_credentials: Path):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "--credentials",
                str(dummy_credentials),
                "--password",
                "topsecret",
                "all-processdata",
            ],
        )
        assert result.exit_code == 2
        assert "password cannot be used with credentials" in result.output

    @pytest.mark.filterwarnings(
        "ignore:--password-file is deprecated. Use --credentials instead."
    )
    def test_crendentials_and_password_file(self, dummy_credentials: Path):
        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "--credentials",
                str(dummy_credentials),
                "--password-file",
                str(dummy_credentials),
                "all-processdata",
            ],
        )

        assert result.exit_code == 2
        assert "password-file cannot be used with credentials" in result.output

    def test_crendentials_and_service_code(
        self, dummy_credentials: Path, tmp_path: Path
    ):
        # As --password-file has a default value, this ensures
        # that no default password-file exists.
        os.chdir(tmp_path)

        runner = CliRunner()
        result = runner.invoke(
            cli,
            [
                "--credentials",
                str(dummy_credentials),
                "--service-code",
                "topsecret",
                "all-processdata",
            ],
        )
        assert result.exit_code == 2
        assert "service_code cannot be used with credentials" in result.output


@only_smoketest
def test_read_process_data(
    credentials: Path,
    session_cache: SessionCache,
    smoketest_config: tuple[str, int, str],
):
    # As --password-file has a default value, this ensures
    # that no default password-file exists.
    os.chdir(credentials.parent)

    host, port, _ = smoketest_config

    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "--host",
            host,
            "--port",
            str(port),
            "--credentials",
            str(credentials),
            "all-processdata",
        ],
    )
    assert result.exit_code == 0
    # check any data which is most likely present on most inverter
    assert "devices:local/Inverter:State" in result.stdout.splitlines()
    assert session_cache.read_session_id() is not None


@only_smoketest
def test_read_settings_data(
    credentials: Path,
    session_cache: SessionCache,
    smoketest_config: tuple[str, int, str],
):
    # As --password-file has a default value, this ensures
    # that no default password-file exists.
    os.chdir(credentials.parent)

    host, port, _ = smoketest_config

    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "--host",
            host,
            "--port",
            str(port),
            "--credentials",
            str(credentials),
            "all-settings",
        ],
    )
    assert result.exit_code == 0
    # check any data which is most likely present on most inverter
    assert "devices:local/Branding:ProductName1" in result.stdout.splitlines()
    assert session_cache.read_session_id() is not None



import asyncio
import sys

from aiohttp import ClientSession

from pykoplenti import ApiClient

"""
Provides a simple example which reads two process data from the plenticore.

Must be called with host and password:
  `python read_process_data.py 192.168.1.100 mysecretpassword`

"""


async def async_main(host, passwd):
    async with ClientSession() as session:
        client = ApiClient(session, host)
        await client.login(passwd)

        data = await client.get_process_data_values(
            "devices:local", ["Inverter:State", "Home_P"]
        )

        device_local = data["devices:local"]
        inverter_state = device_local["Inverter:State"]
        home_p = device_local["Home_P"]

        print(f"Inverter-State: {inverter_state.value}\nHome-P: {home_p.value}\n")


if len(sys.argv) != 3:
    print("Usage: <host> <password>")
    sys.exit(1)

_, host, passwd = sys.argv

asyncio.run(async_main(host, passwd))



import asyncio
import sys

from aiohttp import ClientSession

from pykoplenti import ExtendedApiClient

"""
Provides a simple example which reads virtual process data from the plenticore.

Must be called with host and password:
  `python read_virtual_process_data.py 192.168.1.100 mysecretpassword`

"""


async def async_main(host, passwd):
    async with ClientSession() as session:
        client = ExtendedApiClient(session, host)
        await client.login(passwd)

        data = await client.get_process_data_values("_virt_", "pv_P")

        pv_power = data["_virt_"]["pv_P"]

        print(f"PV power: {pv_power}")


if len(sys.argv) != 3:
    print("Usage: <host> <password>")
    sys.exit(1)

_, host, passwd = sys.argv

asyncio.run(async_main(host, passwd))
